@@ -1,22 +1,23 @@
-;;; Toolbar related functions and commands
-;;; Copyright (C) 1995-1997, 2000, 2001 Kyle E. Jones
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 1, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-(provide 'vm-toolbar)
+;;; vm-toolbar.el --- Toolbar related functions and commands
+;;
+;; Copyright (C) 1995-1997, 2000, 2001 Kyle E. Jones
+;; Copyright (C) 2003-2006 Robert Widhopf-Fenk
+;;
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2 of the License, or
+;; (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License along
+;; with this program; if not, write to the Free Software Foundation, Inc.,
+;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+;;; Code:
 (defvar vm-toolbar-specifier nil)
 
 (defvar vm-toolbar-next-button
@@ -119,6 +120,34 @@ s-expression like this one in your .vm file:
 (or (fboundp 'vm-toolbar-reply-command)
     (fset 'vm-toolbar-reply-command 'vm-followup-include-text))
 
+(defvar vm-toolbar-forward-button
+  [vm-toolbar-forward-icon
+   vm-toolbar-forward-command
+   (vm-toolbar-any-messages-p)
+   "Forward the current message.\n
+The command `vm-toolbar-forward-command' is run, which is normally
+fbound to `vm-forward-message'.
+You can make this button run some other command by using a Lisp
+s-expression like this one in your .vm file:
+   (fset 'vm-toolbar-forward-command 'some-other-command)"])
+(defvar vm-toolbar-forward-icon nil)
+(or (fboundp 'vm-toolbar-forward-command)
+    (fset 'vm-toolbar-forward-command 'vm-forward-message))
+
+(defvar vm-toolbar-followup-button
+  [vm-toolbar-followup-icon
+   vm-toolbar-followup-command
+   (vm-toolbar-any-messages-p)
+   "Follow up the current message.\n
+The command `vm-toolbar-followup-command' is run, which is normally
+fbound to `vm-followup-message'.
+You can make this button run some other command by using a Lisp
+s-expression like this one in your .vm file:
+   (fset 'vm-toolbar-followup-command 'some-other-command)"])
+(defvar vm-toolbar-followup-icon nil)
+(or (fboundp 'vm-toolbar-followup-command)
+    (fset 'vm-toolbar-followup-command 'vm-followup))
+
 (defvar vm-toolbar-compose-button
   [vm-toolbar-compose-icon vm-toolbar-compose-command t
    "Compose a new message.\n
@@ -178,7 +207,7 @@ s-expression like this one in your .vm file:
 VM uses this button to offer help if you're in trouble.
 Under normal circumstances, this button runs `vm-help'.
 If the current folder looks out-of-date relative to its auto-save
-file then this button will run `recover-file'
+file then this button will run `vm-recover-folder'.
 If there is mail waiting in one of the spool files associated
 with the current folder, and the `getmail' button is not on the
 toolbar, this button will run `vm-get-new-mail'.
@@ -188,6 +217,7 @@ will run 'vm-decode-mime-message'."])
 (defvar vm-toolbar-helper-command nil)
 (make-variable-buffer-local 'vm-toolbar-helper-command)
 
+;;;###autoload
 (defun vm-toolbar-helper-command ()
   (interactive)
   (setq this-command vm-toolbar-helper-command)
@@ -214,6 +244,7 @@ s-expression like this one in your .vm file:
 	vm-message-list)
     (error nil)))
 
+;;;###autoload
 (defun vm-toolbar-delete/undelete-message (&optional prefix-arg)
   (interactive "P")
   (vm-follow-summary-cursor)
@@ -226,6 +257,7 @@ s-expression like this one in your .vm file:
 	(call-interactively 'vm-undelete-message)
       (call-interactively 'vm-delete-message))))
 
+;;;###autoload
 (defun vm-toolbar-can-autofile-p ()
   (interactive)
   (condition-case nil
@@ -236,6 +268,7 @@ s-expression like this one in your .vm file:
 	     (vm-auto-select-folder vm-message-pointer vm-auto-folder-alist)))
     (error nil)))
 
+;;;###autoload
 (defun vm-toolbar-autofile-message ()
   (interactive)
   (vm-follow-summary-cursor)
@@ -296,7 +329,7 @@ s-expression like this one in your .vm file:
       (setq vm-toolbar-delete/undelete-icon vm-toolbar-undelete-icon)
     (setq vm-toolbar-delete/undelete-icon vm-toolbar-delete-icon))
   (cond ((vm-toolbar-can-recover-p)
-	 (setq vm-toolbar-helper-command 'recover-file
+	 (setq vm-toolbar-helper-command 'vm-recover-folder
 	       vm-toolbar-helper-icon vm-toolbar-recover-icon))
 	((and (vm-toolbar-mail-waiting-p)
 	      (not (memq 'getmail vm-use-toolbar)))
@@ -325,18 +358,35 @@ s-expression like this one in your .vm file:
 	 (set-specifier vm-toolbar-specifier (cons (current-buffer)
 						   vm-toolbar)))))
 
+(defun vm-toolbar-install-or-uninstall-toolbar ()
+  (and (vm-toolbar-support-possible-p) vm-use-toolbar
+       (vm-toolbar-install-toolbar))
+  (if (and vm-fsfemacs-p (not vm-use-toolbar))
+      (vm-toolbar-fsfemacs-uninstall-toolbar)))
+
+(defcustom vm-toolbar-height nil
+  "*Desired height of the toolbar."
+  :group 'vm
+  :type '(choice (const  :tag "Automatic" nil) integer))
+
 (defun vm-toolbar-install-toolbar ()
+  ;; drag these in now instead of waiting for them to be
+  ;; autoloaded.  the "loading..." messages could come at a bad
+  ;; moment and wipe an important echo area message, like "Auto
+  ;; save file is newer..."
+  (require 'vm-save)
+  (require 'vm-summary)
   (if vm-fsfemacs-p
       (if (not vm-fsfemacs-toolbar-installed-p)
 	  (vm-toolbar-fsfemacs-install-toolbar))
-    (if (not (and (stringp vm-toolbar-pixmap-directory)
-		  (file-directory-p vm-toolbar-pixmap-directory)))
+    (if (not (vm-toolbar-pixmap-directory))
 	(progn
 	  (message "Bad toolbar pixmap directory, can't setup toolbar.")
 	  (sit-for 2))
       (vm-toolbar-initialize)
-      (let ((height (+ 4 (glyph-height (car vm-toolbar-help-icon))))
-	    (width (+ 4 (glyph-width (car vm-toolbar-help-icon))))
+      (let ((height (or vm-toolbar-height
+			(+ 5 (glyph-height (car vm-toolbar-help-icon)))))
+	    (width (+ 5 (glyph-width (car vm-toolbar-help-icon))))
 	    (frame (selected-frame))
 	    (buffer (current-buffer))
 	    (tag-set '(win))
@@ -345,9 +395,9 @@ s-expression like this one in your .vm file:
 	;; glyph-width and glyph-height return 0 at startup sometimes
 	;; use reasonable values if they fail.
 	(if (= width 4)
-	    (setq width 68))
+	    (setq width 38))
 	(if (= height 4)
-	    (setq height 46))
+	    (setq height 38))
 	;; honor user setting of vm-toolbar if they are daring enough
 	;; to set it.
 	(if vm-toolbar
@@ -393,6 +443,8 @@ s-expression like this one in your .vm file:
 			(print . vm-toolbar-print-button)
 			(quit . vm-toolbar-quit-button)
 			(reply . vm-toolbar-reply-button)
+			(forward . vm-toolbar-forward-button)
+			(followup . vm-toolbar-followup-button)
 			(visit . vm-toolbar-visit-button)
 			))
 	(button-list vm-use-toolbar)
@@ -414,64 +466,46 @@ s-expression like this one in your .vm file:
     (nreverse toolbar) ))
 
 (defun vm-toolbar-initialize ()
-  ;; drag these in now instead of waiting for them to be
-  ;; autoloaded.  the "loading..." messages could come at a bad
-  ;; moment and wipe an important echo area message, like "Auto
-  ;; save file is newer..."
-  (require 'vm-save)
-  (require 'vm-summary)
   (cond
    (vm-fsfemacs-p nil)
    ((null vm-toolbar-help-icon)
     (let ((tuples
-	   (if (featurep 'xpm)
-	       (list
-		(if (and (device-on-window-system-p)
-			 (>= (device-bitplanes) 16))
-      '(vm-toolbar-decode-mime-icon "mime-colorful-up.xpm"
-				    "mime-colorful-dn.xpm"
-				    "mime-colorful-xx.xpm")
-   '(vm-toolbar-decode-mime-icon "mime-simple-up.xpm"
-				 "mime-simple-dn.xpm"
-				 "mime-simple-xx.xpm"))
- '(vm-toolbar-next-icon "next-up.xpm" "next-dn.xpm" "next-dn.xpm")
- '(vm-toolbar-previous-icon "previous-up.xpm" "previous-dn.xpm"
-			   "previous-dn.xpm")
- '(vm-toolbar-delete-icon "delete-up.xpm" "delete-dn.xpm" "delete-dn.xpm")
- '(vm-toolbar-undelete-icon "undelete-up.xpm" "undelete-dn.xpm"
-			   "undelete-dn.xpm")
- '(vm-toolbar-autofile-icon "autofile-up.xpm" "autofile-dn.xpm"
-			   "autofile-dn.xpm")
- '(vm-toolbar-getmail-icon "getmail-up.xpm" "getmail-dn.xpm" "getmail-dn.xpm")
- '(vm-toolbar-file-icon "file-up.xpm" "file-dn.xpm" "file-dn.xpm")
- '(vm-toolbar-reply-icon "reply-up.xpm" "reply-dn.xpm" "reply-dn.xpm")
- '(vm-toolbar-compose-icon "compose-up.xpm" "compose-dn.xpm" "compose-dn.xpm")
- '(vm-toolbar-print-icon "print-up.xpm" "print-dn.xpm" "print-dn.xpm")
- '(vm-toolbar-visit-icon "visit-up.xpm" "visit-dn.xpm" "visit-dn.xpm")
- '(vm-toolbar-quit-icon "quit-up.xpm" "quit-dn.xpm" "quit-dn.xpm")
- '(vm-toolbar-help-icon "help-up.xpm" "help-dn.xpm" "help-dn.xpm")
- '(vm-toolbar-recover-icon "recover-up.xpm" "recover-dn.xpm" "recover-dn.xpm")
-	   )
-	       '(
- (vm-toolbar-decode-mime-icon "mime-up.xbm" "mime-dn.xbm" "mime-xx.xbm")
- (vm-toolbar-next-icon "next-up.xbm" "next-dn.xbm" "next-xx.xbm")
- (vm-toolbar-previous-icon "previous-up.xbm" "previous-dn.xbm"
-			   "previous-xx.xbm")
- (vm-toolbar-delete-icon "delete-up.xbm" "delete-dn.xbm" "delete-xx.xbm")
- (vm-toolbar-undelete-icon "undelete-up.xbm" "undelete-dn.xbm"
-			   "undelete-xx.xbm")
- (vm-toolbar-autofile-icon "autofile-up.xbm" "autofile-dn.xbm"
-			   "autofile-xx.xbm")
- (vm-toolbar-getmail-icon "getmail-up.xbm" "getmail-dn.xbm" "getmail-xx.xbm")
- (vm-toolbar-file-icon "file-up.xbm" "file-dn.xbm" "file-xx.xbm")
- (vm-toolbar-reply-icon "reply-up.xbm" "reply-dn.xbm" "reply-xx.xbm")
- (vm-toolbar-compose-icon "compose-up.xbm" "compose-dn.xbm" "compose-xx.xbm")
- (vm-toolbar-print-icon "print-up.xbm" "print-dn.xbm" "print-xx.xbm")
- (vm-toolbar-visit-icon "visit-up.xbm" "visit-dn.xbm" "visit-xx.xbm")
- (vm-toolbar-quit-icon "quit-up.xbm" "quit-dn.xbm" "quit-xx.xbm")
- (vm-toolbar-help-icon "help-up.xbm" "help-dn.xbm" "help-xx.xpm")
- (vm-toolbar-recover-icon "recover-up.xbm" "recover-dn.xbm" "recover-xx.xpm")
-	   )))
+           (list
+            '(vm-toolbar-decode-mime-icon
+              "mime-up.xpm" "mime-dn.xpm" "mime-xx.xpm")
+            '(vm-toolbar-next-icon
+              "next-up.xpm" "next-dn.xpm" "next-dn.xpm")
+            '(vm-toolbar-previous-icon
+              "previous-up.xpm" "previous-dn.xpm" "previous-dn.xpm")
+            '(vm-toolbar-delete-icon
+              "delete-up.xpm" "delete-dn.xpm" "delete-dn.xpm")
+            '(vm-toolbar-undelete-icon
+              "undelete-up.xpm" "undelete-dn.xpm" "undelete-dn.xpm")
+            '(vm-toolbar-autofile-icon
+              "autofile-up.xpm" "autofile-dn.xpm" "autofile-dn.xpm")
+            '(vm-toolbar-getmail-icon
+              "getmail-up.xpm" "getmail-dn.xpm" "getmail-dn.xpm")
+            '(vm-toolbar-file-icon
+              "file-up.xpm" "file-dn.xpm" "file-dn.xpm")
+            '(vm-toolbar-reply-icon
+              "reply-up.xpm" "reply-dn.xpm" "reply-dn.xpm")
+            '(vm-toolbar-forward-icon
+              "forward-up.xpm" "forward-dn.xpm" "forward-dn.xpm")
+            '(vm-toolbar-followup-icon
+              "followup-up.xpm" "followup-dn.xpm" "followup-dn.xpm")
+            '(vm-toolbar-compose-icon
+              "compose-up.xpm" "compose-dn.xpm" "compose-dn.xpm")
+            '(vm-toolbar-print-icon
+              "print-up.xpm" "print-dn.xpm" "print-dn.xpm")
+            '(vm-toolbar-visit-icon
+              "visit-up.xpm" "visit-dn.xpm" "visit-dn.xpm")
+            '(vm-toolbar-quit-icon
+              "quit-up.xpm" "quit-dn.xpm" "quit-dn.xpm")
+            '(vm-toolbar-help-icon
+              "help-up.xpm" "help-dn.xpm" "help-dn.xpm")
+            '(vm-toolbar-recover-icon
+              "recover-up.xpm" "recover-dn.xpm" "recover-dn.xpm")
+            ))
 	  tuple files var)
       (while tuples
 	(setq tuple (car tuples)
@@ -481,7 +515,7 @@ s-expression like this one in your .vm file:
 		  (function
 		   (lambda (f)
 		     (make-glyph
-		      (expand-file-name f vm-toolbar-pixmap-directory))))
+		      (expand-file-name f (vm-toolbar-pixmap-directory)))))
 		  files))
 	(setq tuples (cdr tuples))))))
   (setq vm-toolbar-delete/undelete-icon vm-toolbar-delete-icon)
@@ -490,13 +524,14 @@ s-expression like this one in your .vm file:
   (setq vm-toolbar-helper-icon vm-toolbar-help-icon)
   (setq-default vm-toolbar-helper-icon vm-toolbar-help-icon))
 
+(defun vm-toolbar-fsfemacs-uninstall-toolbar ()
+  (define-key vm-mode-map [toolbar] nil)
+  (setq vm-fsfemacs-toolbar-installed-p nil))
+
 (defun vm-toolbar-fsfemacs-install-toolbar ()
   (let ((button-list (reverse vm-use-toolbar))
-	(dir vm-toolbar-pixmap-directory)
-	(extension (if (and (display-color-p)
-			    (image-type-available-p 'xpm))
-		       "xpm"
-		     "xbm"))
+	(dir (vm-toolbar-pixmap-directory))
+	(extension "xpm")
 	item t-spec sym name images)
     (defvar tool-bar-map)
     ;; hide the toolbar entries that are in the global keymap so
@@ -518,30 +553,30 @@ s-expression like this one in your .vm file:
 	     ;; can't do separators in FSF Emacs
 	     t)
 	    ((memq sym '(autofile compose file getmail
-			 mime next previous print quit reply visit))
+			 mime next previous print quit
+			 reply followup forward visit))
 	     (setq t-spec (symbol-value
 			   (intern (format "vm-toolbar-%s-button"
 					   (if (eq sym 'mime)
 					       'decode-mime
 					     sym)))))
-	     (if (and (eq sym 'mime) (string= extension "xpm"))
-		 (setq name "mime-colorful")
-	       (setq name (symbol-name sym)))
+             (setq name (symbol-name sym))
 	     (setq images (vm-toolbar-make-fsfemacs-toolbar-image-spec
-			   name extension dir))
+			   name extension dir
+			   (if (eq sym 'mime) nil 'heuristic)))
 	     (setq item
 		   (list 'menu-item
 			 (aref t-spec 3)
 			 (aref t-spec 1)
 			 ':enable (aref t-spec 2)
-			 ':button '(:toggle nil)
+;			 ':button '(:toggle nil)
 			 ':image images))
 	     (define-key vm-mode-map (vector 'tool-bar sym) item))
 	    ((eq sym 'delete/undelete)
 	     (setq t-spec vm-toolbar-delete/undelete-button)
 	     (setq name "delete")
 	     (setq images (vm-toolbar-make-fsfemacs-toolbar-image-spec
-			   name extension dir))
+			   name extension dir 'heuristic))
 	     (setq item
 		   (list 'menu-item
 			 (aref t-spec 3)
@@ -549,12 +584,12 @@ s-expression like this one in your .vm file:
 			 ':visible '(eq vm-toolbar-delete/undelete-icon
 					vm-toolbar-delete-icon)
 			 ':enable (aref t-spec 2)
-			 ':button '(:toggle nil)
+;			 ':button '(:toggle nil)
 			 ':image images))
 	     (define-key vm-mode-map (vector 'tool-bar 'delete) item)
 	     (setq name "undelete")
 	     (setq images (vm-toolbar-make-fsfemacs-toolbar-image-spec
-			   name extension dir))
+			   name extension dir 'heuristic))
 	     (setq item
 		   (list 'menu-item
 			 (aref t-spec 3)
@@ -562,26 +597,26 @@ s-expression like this one in your .vm file:
 			 ':visible '(eq vm-toolbar-delete/undelete-icon
 					vm-toolbar-undelete-icon)
 			 ':enable (aref t-spec 2)
-			 ':button '(:toggle nil)
+;			 ':button '(:toggle nil)
 			 ':image images))
 	     (define-key vm-mode-map (vector 'tool-bar 'undelete) item))
 	    ((eq sym 'help)
 	     (setq t-spec vm-toolbar-help-button)
 	     (setq name "help")
 	     (setq images (vm-toolbar-make-fsfemacs-toolbar-image-spec
-			   name extension dir))
+			   name extension dir 'heuristic))
 	     (setq item
 		   (list 'menu-item
 			 (aref t-spec 3)
 			 (aref t-spec 1)
 			 ':visible '(eq vm-toolbar-helper-command 'vm-help)
 			 ':enable (aref t-spec 2)
-			 ':button '(:toggle nil)
+;			 ':button '(:toggle nil)
 			 ':image images))
 	     (define-key vm-mode-map (vector 'tool-bar 'help-help) item)
 	     (setq name "recover")
 	     (setq images (vm-toolbar-make-fsfemacs-toolbar-image-spec
-			   name extension dir))
+			   name extension dir 'heuristic))
 	     (setq item
 		   (list 'menu-item
 			 (aref t-spec 3)
@@ -589,12 +624,12 @@ s-expression like this one in your .vm file:
 			 ':visible '(eq vm-toolbar-helper-command
 					'recover-file)
 			 ':enable (aref t-spec 2)
-			 ':button '(:toggle nil)
+;			 ':button '(:toggle nil)
 			 ':image images))
 	     (define-key vm-mode-map (vector 'tool-bar 'help-recover) item)
 	     (setq name "getmail")
 	     (setq images (vm-toolbar-make-fsfemacs-toolbar-image-spec
-			   name extension dir))
+			   name extension dir 'heuristic))
 	     (setq item
 		   (list 'menu-item
 			 (aref t-spec 3)
@@ -602,14 +637,12 @@ s-expression like this one in your .vm file:
 			 ':visible '(eq vm-toolbar-helper-command
 					'vm-get-new-mail)
 			 ':enable (aref t-spec 2)
-			 ':button '(:toggle nil)
+;			 ':button '(:toggle nil)
 			 ':image images))
 	     (define-key vm-mode-map (vector 'tool-bar 'help-getmail) item)
-	     (if (string= extension "xpm")
-		 (setq name "mime-colorful")
-	       (setq name "mime"))
+             (setq name "mime")
 	     (setq images (vm-toolbar-make-fsfemacs-toolbar-image-spec
-			   name extension dir))
+			   name extension dir nil))
 	     (setq item
 		   (list 'menu-item
 			 (aref t-spec 3)
@@ -617,39 +650,39 @@ s-expression like this one in your .vm file:
 			 ':visible '(eq vm-toolbar-helper-command
 					'vm-decode-mime-message)
 			 ':enable (aref t-spec 2)
-			 ':button '(:toggle nil)
+;			 ':button '(:toggle nil)
 			 ':image images))
 	     (define-key vm-mode-map (vector 'tool-bar 'help-mime) item)))
       (setq button-list (cdr button-list))))
   (setq vm-fsfemacs-toolbar-installed-p t))
 
-(defun vm-toolbar-make-fsfemacs-toolbar-image-spec (name extension dir)
+(defun vm-toolbar-make-fsfemacs-toolbar-image-spec (name extension dir mask)
   (if (string= extension "xpm")
       (vector
        (list 'image
 	     ':type (intern extension)
-	     ':mask 'heuristic
+	     ':mask mask
 	     ':file (expand-file-name
 		     (format "%s-dn.%s"
 			     name extension)
 		     dir))
        (list 'image
 	     ':type (intern extension)
-	     ':mask 'heuristic
+	     ':mask mask
 	     ':file (expand-file-name
 		     (format "%s-up.%s"
 			     name extension)
 		     dir))
        (list 'image
 	     ':type (intern extension)
-	     ':mask 'heuristic
+	     ':mask mask
 	     ':file (expand-file-name
 		     (format "%s-dn.%s"
 			     name extension)
 		     dir))
        (list 'image
 	     ':type (intern extension)
-	     ':mask 'heuristic
+	     ':mask mask
 	     ':file (expand-file-name
 		     (format "%s-dn.%s"
 			     name extension)
@@ -657,29 +690,33 @@ s-expression like this one in your .vm file:
     (vector
      (list 'image
 	   ':type (intern extension)
-	   ':mask 'heuristic
+	   ':mask mask
 	   ':file (expand-file-name
 		   (format "%s-dn.%s"
 			   name extension)
 		   dir))
      (list 'image
 	   ':type (intern extension)
-	   ':mask 'heuristic
+	   ':mask mask
 	   ':file (expand-file-name
 		   (format "%s-up.%s"
 			   name extension)
 		   dir))
      (list 'image
 	   ':type (intern extension)
-	   ':mask 'heuristic
+	   ':mask mask
 	   ':file (expand-file-name
 		   (format "%s-xx.%s"
 			   name extension)
 		   dir))
      (list 'image
 	   ':type (intern extension)
-	   ':mask 'heuristic
+	   ':mask mask
 	   ':file (expand-file-name
 		   (format "%s-xx.%s"
 			   name extension)
 		   dir)))))
+
+(provide 'vm-toolbar)
+
+;;; vm-toolbar.el ends here
